/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/

#ifndef TOOLMATRIX_H
#define TOOLMATRIX_H

#include <glib.h>

G_BEGIN_DECLS

/**
 * matrix_productMatrix:
 * @matRes: an array of floating point values of size 3x3 ;
 * @matA: an array of floating point values of size 3x3 ;
 * @matB: an array of floating point values of size 3x3.
 *
 * Compute the mathematical product between @matA and @matB and
 * put the result matrix in @matRes.
 */
void matrix_productMatrix(float matRes[3][3], float matA[3][3], float matB[3][3]);
/**
 * matrix_productVector:
 * @vectRes: an array of floating point values of size 3 ;
 * @mat: an array of floating point values of size 3x3 ;
 * @vect: an array of floating point values of size 3.
 *
 * Compute the mathematical product between @matA and @vect and
 * put the result vector in @vectRes.
 */
void matrix_productVector(float vectRes[3], float mat[3][3], float vect[3]);
gboolean matrix_invert(float inv[3][3], float mat[3][3]);

/**
 * matrix_reducePrimitiveVectors:
 * @reduced: a storage for 6 floating point values ;
 * @full: a full 3x3 matrix to be transformed.
 *
 * This routine transforms the given matrix @full into a reduced array
 * used by V_Sim to store box definition.
 *
 * Returns: FALSE if the given matrix is planar.
 */
gboolean matrix_reducePrimitiveVectors(double reduced[6], double full[3][3]);


/**
 * matrix_cartesianToSpherical:
 * @spherical: an allocated array of 3 floating point values to store the result ;
 * @cartesian: an allocated array of 3 floating point values to read the input.
 *
 * A method to transform cartesian coordinates in spherical coordinates (radius, phi and theta).
 */
void matrix_cartesianToSpherical(float *spherical, float *cartesian);
/**
 * matrix_sphericalToCartesian:
 * @cartesian: an allocated array of 3 floating point values to store the result ;
 * @spherical: an allocated array of 3 floating point values to read the input.
 *
 * A method to transform spherical coordinates (radius, phi and theta) to cartesian coordinates.
 */
void matrix_sphericalToCartesian(float *cartesian, float *spherical);

/**
 * matrix_scaleflag:
 * @linear: a linear convertion from [min,max] to [0,1] ;
 * @logarithm: a logarithmic transformation from [min,max] to [0,1], the
 * formula is -(f(x) - f(m) / f(m) where f(x) =
 * ln((x-xmin)/(xmax-xmin)) ;
 * @zeroCentredLog: a logarithmic transformation for data that are
 * zero centred, the formula is
 * 0.5+s*(log(MAX*SEUIL)-log(max(abs(x),MAX*SEUIL)))/(2*log(SEUIL))
 * where s is the sign, max=max(xmax,-xmin) and seuil a parameter
 * (1e-5).
 * @n_scaleflag: number of available scale functions.
 *
 * Flag used to specify the transformation for scalarFieldDraw_map()
 * routine.
 *
 * Since: 3.4
 */
typedef enum
  {
    linear,
    logarithm,
    zeroCentredLog,
    n_scaleflag
  } matrix_scaleflag;
/**
 * matrixGet_scaleVal:
 * @x: the initial value ;
 * @minmax: the boundaries for the @x argument ;
 * @param: a parameter.
 *
 * Transform @x into [0;1] using the given @minmax values.
 *
 * Returns: a value into [0;1].
 *
 * Since: 3.4
 */
typedef double (*matrixGet_scaleVal)(double x, double minmax[2], double param);
/**
 * matrixGet_linear:
 * @x: the initial value ;
 * @minmax: the boundaries for the @x argument ;
 * @param: a parameter (unused).
 *
 * Transform @x into [0;1] with a linear scale.
 *
 * Returns: a value into [0;1].
 *
 * Since: 3.5
 */
double matrixGet_linear(double x, double minmax[2], double param);
/**
 * matrixGet_logarithm:
 * @x: the initial value ;
 * @minmax: the boundaries for the @x argument ;
 * @param: the second minimum value, used as reference for the log scale.
 *
 * Transform @x into [0;1] with a log scale.
 *
 * Returns: a value into [0;1].
 *
 * Since: 3.5
 */
double matrixGet_logarithm(double x, double minmax[2], double param);
/**
 * matrixGet_zeroCenteredLog:
 * @x: the initial value ;
 * @minmax: the boundaries for the @x argument ;
 * @param: the result of MAX(@minmax[1], -@minax[0]).
 *
 * Transform @x into [0;1] with a log scale with zero centred values.
 *
 * Returns: a value into [0;1].
 *
 * Since: 3.5
 */
double matrixGet_zeroCenteredLog(double x, double minmax[2], double param);

/**
 * matrixGet_invLinear:
 * @x: the initial value ;
 * @minmax: the boundaries for the @x argument ;
 * @param: a parameter (unused).
 *
 * Reverse function for matrixGet_linear().
 *
 * Returns: a value into [0;1].
 *
 * Since: 3.5
 */
double matrixGet_invLinear(double x, double minmax[2], double param);
/**
 * matrixGet_invLogarithm:
 * @x: the initial value ;
 * @minmax: the boundaries for the @x argument ;
 * @param: the second minimum value, used as reference for the log scale.
 *
 * Reverse function for matrixGet_logarithm().
 *
 * Returns: a value into [0;1].
 *
 * Since: 3.5
 */
double matrixGet_invLogarithm(double x, double minmax[2], double param);
/**
 * matrixGet_invZeroCenteredLog:
 * @x: the initial value ;
 * @minmax: the boundaries for the @x argument ;
 * @param: the result of MAX(@minmax[1], -@minax[0]).
 *
 * Reverse function for matrixGet_zeroCenteredLog().
 *
 * Returns: a value into [0;1].
 *
 * Since: 3.5
 */
double matrixGet_invZeroCenteredLog(double x, double minmax[2], double param);

gboolean matrixGet_inter2D(float *lambda,
			   float a[2], float b[2], float A[2], float B[2]);
gboolean matrixGet_inter2DFromList(float i[2], float *lambda,
				   float a[2], float b[2], GList *set);

/**
 * matrixInit:
 *
 * This method is used by V_Sim internally and should not be called.
 *
 * Since: 3.5
 */
void matrixInit();


G_END_DECLS

#endif
